Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;
-------------------------------------------------------------------------------
-- MODULE DESCRIPTION, March 30, 2015


entity BiasCurve_MemorySelect is 
port( 
		Proc_WR_A_nB		: in std_logic; -- controlled by BC curve player
		Enable_VME_Ctrl	: in std_logic;
		VME_RD_A_nB			: in std_logic; -- controlled by a VME control register
		VME_WR_A_nB    	: in std_logic; -- used for memory testing
		
		UPPER_WR_BC			: in std_logic;
		BC_VME_Data_OutA	: in std_logic_vector(15 downto 0);
		BC_VME_Data_OutB	: in std_logic_vector(15 downto 0);
		
		BC_PROC_Data_OutA	: in std_logic_vector(15 downto 0);
		BC_PROC_Data_OutB	: in std_logic_vector(15 downto 0);
		
		UPPER_WR_BC_A	: out std_logic;
		UPPER_WR_BC_B	: out std_logic;

		BC_VME_Data_Outx	: out std_logic_vector(31 downto 0);
		BC_PROC_Data_Outx	: out std_logic_vector(31 downto 0)
		
	);
end BiasCurve_MemorySelect;
	
architecture main of BiasCurve_MemorySelect is	

signal wr_A_nB	: std_logic;

signal proc_RD_A_nB : std_logic;

--=====================================================================
--				B  E  G  I  N 
--=====================================================================
begin
process (Proc_WR_A_nB, Enable_VME_Ctrl, VME_WR_A_nB)
begin
	if  ( Enable_VME_Ctrl = '1') then	wr_A_nB <= VME_WR_A_nB;
	else											wr_A_nB <= Proc_WR_A_nB;
	end if;
end process;


process (VME_RD_A_nB, BC_VME_Data_OutA, BC_VME_Data_OutB)
begin
	if  (VME_RD_A_nB = '1') then		BC_VME_Data_Outx <= X"0000" & BC_VME_Data_OutA;
	else										BC_VME_Data_Outx <= X"0000" & BC_VME_Data_OutB;
	end if;
end process;

process (wr_A_nB, UPPER_WR_BC)
begin
	if  (wr_A_nB = '1') then		UPPER_WR_BC_A <= UPPER_WR_BC;
											UPPER_WR_BC_B <= '0';
											
	else									UPPER_WR_BC_B <= UPPER_WR_BC;
											UPPER_WR_BC_A <= '0';
	end if;
end process;

proc_RD_A_nB <= not Proc_WR_A_nB;

process (proc_RD_A_nB, BC_PROC_Data_OutA, BC_PROC_Data_OutB)
begin
	if  (proc_RD_A_nB = '1') then	BC_PROC_Data_Outx <= X"0000" & BC_PROC_Data_OutA;
	else									BC_PROC_Data_Outx <= X"0000" & BC_PROC_Data_OutB;
	end if;
end process;


end;