Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

-------------------------------------------------------------------------------
--Version 2 4/5/2017: in this version we changed how the rshift_cnt is reset.

entity InterFPGA_Slave_v2 is 
	port( 
			clk				: in std_logic;
			nreset			: in std_logic;
			XMT_SR_SYNC		: in std_logic;
			XMT_SR_CLK		: in std_logic;
			RCV_SR_SYNC		: in std_logic;
			RCV_SR_CLK		: in std_logic;
			
			Parallel_D_out	: in std_logic_vector(31 downto 0);
			Serial_D_in		: in std_logic_vector(7 downto 0);
			Serial_A_in		: in std_logic_vector(4 downto 0);

			WR_lower_mem	: in std_logic;
			LD_lower_mem	: in std_logic;
			RD_lower_sr		: in std_logic;
			ENA_shift_in	: in std_logic;
                           
			RCV_CLK_IN		: in std_logic;
			
			Parallel_D_in	: out std_logic_vector(31 downto 0);
			Parallel_A_in	: out std_logic_vector(19 downto 0);
			Serial_D_out	: out std_logic_vector(7 downto 0);
			
			WR_mem			: out std_logic;
			
			XMT_CLK_TEST2	: out std_logic;
			load_test		: out std_logic;
			
			RCV_CLK_OUT		: out std_logic
		);
	end InterFPGA_Slave_v2 ;
	
architecture main of InterFPGA_Slave_v2  is	

-- Shift Register Signals -----------------------------------------------
signal D_XMT_SR_0 		: std_logic_vector(3 downto 0); 
signal D_XMT_SR_1 		: std_logic_vector(3 downto 0); 
signal D_XMT_SR_2 		: std_logic_vector(3 downto 0); 
signal D_XMT_SR_3 		: std_logic_vector(3 downto 0); 
signal D_XMT_SR_4 		: std_logic_vector(3 downto 0); 
signal D_XMT_SR_5 		: std_logic_vector(3 downto 0); 
signal D_XMT_SR_6 		: std_logic_vector(3 downto 0); 
signal D_XMT_SR_7 		: std_logic_vector(3 downto 0); 

signal D_RCV_SR_0 		: std_logic_vector(3 downto 0);
signal D_RCV_SR_1 		: std_logic_vector(3 downto 0);
signal D_RCV_SR_2 		: std_logic_vector(3 downto 0);
signal D_RCV_SR_3 		: std_logic_vector(3 downto 0);
signal D_RCV_SR_4 		: std_logic_vector(3 downto 0);
signal D_RCV_SR_5 		: std_logic_vector(3 downto 0);
signal D_RCV_SR_6 		: std_logic_vector(3 downto 0);
signal D_RCV_SR_7 		: std_logic_vector(3 downto 0);
                

signal A_RCV_SR_0 		: std_logic_vector(3 downto 0); 
signal A_RCV_SR_1 		: std_logic_vector(3 downto 0); 
signal A_RCV_SR_2 		: std_logic_vector(3 downto 0); 
signal A_RCV_SR_3 		: std_logic_vector(3 downto 0); 
signal A_RCV_SR_4 		: std_logic_vector(3 downto 0); 

-- Shift Register Control Signals ----------------------------------------
signal XMT_CLK			: std_logic;
signal load				: std_logic;

signal syncd_ENA_shift	: std_logic;
signal shift_cnt		: std_logic_vector(4 downto 0);
signal rshift_cnt		: std_logic_vector(4 downto 0);
		
--========================================================================
--========================================================================
--                    B E G I N 
--========================================================================
--========================================================================	
begin
	----------------------------------------------------------------------
	-- This process produces four transmitter shift register clock pulses 
	-- for the transmitter shift register in this FPGA.  The pulses are four 
	-- XMT_SR_CLK clock periods gated by a 4 clock wide LED type pulse 
	-- syncronized by XMT_SR_SYNC.  The trigger for all of this is ena_shift.
	--
	-- Signal ena_shift is produced by the state machine which is clocked by
	-- Logic_clk (clk).  
	--
	-- Four receiver shift register clock pulses are generated in a like manner.
	-- These pulses are used in the other FPGA's receive shift register.  Receiver
	-- shift register pulses are generated in the other FPGA for the receiver 
	-- shift register in this FPGA.
	--
	-- The state machine clock, XMT_SR_SYNC, XMT_SR_CLK, RCV_SR_SYNC, RCV_SR_CLK 
	-- are all generated by the same PLL and can be shifted in phase w.r.t. one 
	-- another.  The XMT_SR_SYNC clock is shifted w.r.t. the state machine clock
	-- so that the rising edge of the ena_shift signal is registered by the 
	-- rising edge of XMT_SR_SYNC before the rising edge of RCV_SR_SYNC.  The
	-- sync clocks determine the timing of the transmit clock gate and the 
	-- receive clock gate.  Hence RCV_SR_SYNC is delayed w.r.t. the XMT_SR_SYNC
	-- so as to optimize the setup and hold times for the data arriving at the
	-- receiver shift register from the transmitter shift register.  Then finally
	-- XMT_SR_CLK and RCV_SR_CLK can be phased shifted w.r.t. their associated
	-- gating signals so as not to clip either the first or last of the four
	-- shift register clock pulses.
	
	process(clk, nreset, ENA_shift_in)
	begin
		if    (nreset = '0')   then syncd_ENA_shift <= '0';
		elsif rising_edge(clk) then syncd_ENA_shift <= ENA_shift_in;
		end if;
	end process;
	--------------------------------------------------------
	
	process (nreset, XMT_SR_SYNC, shift_cnt, syncd_ENA_shift)
	begin
		if (nreset = '0')  then 					shift_cnt(4 downto 0) <= "00000";			
		elsif rising_edge(XMT_SR_SYNC) then			
			if   (syncd_ENA_shift = '1') then		-- Changed 4/5/2017 @#$%@#%@#$%#@$%@#$
															shift_cnt(0)          <= '1';
															shift_cnt(4 downto 1) <= shift_cnt(3 downto 0);
															
			else											shift_cnt(4 downto 0) <= "00000";		
			end if;
		end if;
	end process;
		-- create four gated sr clock pulses for the transmitter in this fpga
		XMT_CLK <= XMT_SR_CLK AND ((shift_cnt(0) AND NOT shift_cnt(4)) OR load);
		XMT_CLK_TEST2 <= XMT_CLK;
	--------------------------------------------------------

	process (nreset, LD_lower_mem, XMT_SR_SYNC)
	begin
		if (nreset = '0') then 
			load <= '0';
		elsif rising_edge(XMT_SR_SYNC) then
			load <= LD_lower_mem; --sync load_sr to XMT_SR_SYNC
		end if;
	end process;
	--------------------------------------------------------
		load_test <= load;		
	
	-------------------------------------------------------------------------------	

	process (nreset, RCV_SR_SYNC, syncd_ENA_shift)
	begin
		if (nreset = '0')  then 				rshift_cnt(4 downto 0) <= "00000";			
		elsif rising_edge(RCV_SR_SYNC) then
			if(syncd_ENA_shift = '1') then   -- Change 4/5/2017 @#$%@#%@#$%#@$%@#$
														rshift_cnt(0)          <= '1';
														rshift_cnt(4 downto 1) <= rshift_cnt(3 downto 0);
			
			else										rshift_cnt(4 downto 0) <= "00000";
			end if;
		end if;
	end process;
		-- create four gated sr clock pulses for the receiver in the other fpga
		RCV_CLK_OUT <= RCV_SR_CLK AND (rshift_cnt(0) AND NOT rshift_cnt(4));		
	
-------------------------------------------------------------------------------
-- Eight Data Transmit Shift Registers
	process
	(XMT_CLK, nreset, D_XMT_SR_0, D_XMT_SR_1, D_XMT_SR_2, D_XMT_SR_3, 
		D_XMT_SR_4, D_XMT_SR_5, D_XMT_SR_6, D_XMT_SR_7, Parallel_D_out)
	begin
		if(nreset = '0')  then 
				D_XMT_SR_0 <= "0000";
				D_XMT_SR_1 <= "0000";
				D_XMT_SR_2 <= "0000";
				D_XMT_SR_3 <= "0000";
				D_XMT_SR_4 <= "0000";
				D_XMT_SR_5 <= "0000";
				D_XMT_SR_6 <= "0000";
				D_XMT_SR_7 <= "0000";
				
		elsif rising_edge(XMT_CLK) then
			if (load = '1') then			
				D_XMT_SR_0(3 downto 0) <= Parallel_D_out(3 downto 0);
				D_XMT_SR_1(3 downto 0) <= Parallel_D_out(7 downto 4);
				D_XMT_SR_2(3 downto 0) <= Parallel_D_out(11 downto 8);
				D_XMT_SR_3(3 downto 0) <= Parallel_D_out(15 downto 12);
				D_XMT_SR_4(3 downto 0) <= Parallel_D_out(19 downto 16);
				D_XMT_SR_5(3 downto 0) <= Parallel_D_out(23 downto 20);
				D_XMT_SR_6(3 downto 0) <= Parallel_D_out(27 downto 24);
				D_XMT_SR_7(3 downto 0) <= Parallel_D_out(31 downto 28);
				               
			else				
				D_XMT_SR_0(3 downto 1) <= D_XMT_SR_0(2 downto 0);
				D_XMT_SR_1(3 downto 1) <= D_XMT_SR_1(2 downto 0);
				D_XMT_SR_2(3 downto 1) <= D_XMT_SR_2(2 downto 0);
				D_XMT_SR_3(3 downto 1) <= D_XMT_SR_3(2 downto 0);
				D_XMT_SR_4(3 downto 1) <= D_XMT_SR_4(2 downto 0);
				D_XMT_SR_5(3 downto 1) <= D_XMT_SR_5(2 downto 0);
				D_XMT_SR_6(3 downto 1) <= D_XMT_SR_6(2 downto 0);
				D_XMT_SR_7(3 downto 1) <= D_XMT_SR_7(2 downto 0);
				
				D_XMT_SR_0(0) <= D_XMT_SR_0(3); -- Data wraps around ...
				D_XMT_SR_1(0) <= D_XMT_SR_1(3);
				D_XMT_SR_2(0) <= D_XMT_SR_2(3);
				D_XMT_SR_3(0) <= D_XMT_SR_3(3);
				D_XMT_SR_4(0) <= D_XMT_SR_4(3);
				D_XMT_SR_5(0) <= D_XMT_SR_5(3);
				D_XMT_SR_6(0) <= D_XMT_SR_6(3);
				D_XMT_SR_7(0) <= D_XMT_SR_7(3);
			end if;             
		end if;
	end process;	
				Serial_D_out(0) <= D_XMT_SR_0(3); -- serial data outputs
				Serial_D_out(1) <= D_XMT_SR_1(3);
				Serial_D_out(2) <= D_XMT_SR_2(3);
				Serial_D_out(3) <= D_XMT_SR_3(3);
				Serial_D_out(4) <= D_XMT_SR_4(3);
				Serial_D_out(5) <= D_XMT_SR_5(3);
				Serial_D_out(6) <= D_XMT_SR_6(3);
				Serial_D_out(7) <= D_XMT_SR_7(3);

-------------------------------------------------------------------------
-- Eight Data Receiver Shift Register

	process
	(RCV_CLK_IN, nreset, D_RCV_SR_0, D_RCV_SR_1, D_RCV_SR_2, D_RCV_SR_3, 
		D_RCV_SR_4, D_RCV_SR_5, D_RCV_SR_6, D_RCV_SR_7, Serial_D_in)
	begin
		if(nreset = '0')  then 
			D_RCV_SR_0 <= "0000";
			D_RCV_SR_1 <= "0000";
			D_RCV_SR_2 <= "0000";
			D_RCV_SR_3 <= "0000";
			D_RCV_SR_4 <= "0000";
			D_RCV_SR_5 <= "0000";
			D_RCV_SR_6 <= "0000";
			D_RCV_SR_7 <= "0000";
				
		elsif rising_edge(RCV_CLK_IN) then
			D_RCV_SR_0(0) <= Serial_D_in(0); -- serial data in
			D_RCV_SR_1(0) <= Serial_D_in(1);
			D_RCV_SR_2(0) <= Serial_D_in(2);
			D_RCV_SR_3(0) <= Serial_D_in(3);
			D_RCV_SR_4(0) <= Serial_D_in(4);
			D_RCV_SR_5(0) <= Serial_D_in(5);
			D_RCV_SR_6(0) <= Serial_D_in(6);
			D_RCV_SR_7(0) <= Serial_D_in(7);
			                
			D_RCV_SR_0(3 downto 1) <= D_RCV_SR_0(2 downto 0);
			D_RCV_SR_1(3 downto 1) <= D_RCV_SR_1(2 downto 0);
			D_RCV_SR_2(3 downto 1) <= D_RCV_SR_2(2 downto 0);
			D_RCV_SR_3(3 downto 1) <= D_RCV_SR_3(2 downto 0);
			D_RCV_SR_4(3 downto 1) <= D_RCV_SR_4(2 downto 0);
			D_RCV_SR_5(3 downto 1) <= D_RCV_SR_5(2 downto 0);
			D_RCV_SR_6(3 downto 1) <= D_RCV_SR_6(2 downto 0);
			D_RCV_SR_7(3 downto 1) <= D_RCV_SR_7(2 downto 0);
			
		end if;
	end process;	
					
-------------------------------------------------------------------------
-- Parallel Data Out of the Data Receiver Shift Registers					  
	process (clk, RD_lower_sr, D_RCV_SR_0, D_RCV_SR_1, D_RCV_SR_2, 
			 D_RCV_SR_3, D_RCV_SR_4, D_RCV_SR_5, D_RCV_SR_6, D_RCV_SR_7)
	begin
		if rising_edge(clk) then
			if (RD_lower_sr = '1') then
				Parallel_D_in(3 downto 0)   <= D_RCV_SR_0(3 downto 0);
				Parallel_D_in(7 downto 4)   <= D_RCV_SR_1(3 downto 0);
				Parallel_D_in(11 downto 8)  <= D_RCV_SR_2(3 downto 0);
				Parallel_D_in(15 downto 12) <= D_RCV_SR_3(3 downto 0);
				Parallel_D_in(19 downto 16) <= D_RCV_SR_4(3 downto 0);
				Parallel_D_in(23 downto 20) <= D_RCV_SR_5(3 downto 0);
				Parallel_D_in(27 downto 24) <= D_RCV_SR_6(3 downto 0);
				Parallel_D_in(31 downto 28) <= D_RCV_SR_7(3 downto 0);
				
			end if; 
		end if;		                   
	end process;	
	
--=======================================================================
-- Five Address Receive Shift Registers				  
	process
	(RCV_CLK_IN, nreset, A_RCV_SR_0, A_RCV_SR_1, A_RCV_SR_2, 
			 A_RCV_SR_3, A_RCV_SR_4, Serial_A_in)
	begin
		if(nreset = '0')  then 
			A_RCV_SR_0 <= "0000";
			A_RCV_SR_1 <= "0000";
			A_RCV_SR_2 <= "0000";
			A_RCV_SR_3 <= "0000";
			A_RCV_SR_4 <= "0000";
				
		elsif rising_edge(RCV_CLK_IN) then
			A_RCV_SR_0(0) <= Serial_A_in(0);
			A_RCV_SR_1(0) <= Serial_A_in(1);
			A_RCV_SR_2(0) <= Serial_A_in(2);
			A_RCV_SR_3(0) <= Serial_A_in(3);
			A_RCV_SR_4(0) <= Serial_A_in(4);
			
			A_RCV_SR_0(3 downto 1) <= A_RCV_SR_0(2 downto 0);
			A_RCV_SR_1(3 downto 1) <= A_RCV_SR_1(2 downto 0);
			A_RCV_SR_2(3 downto 1) <= A_RCV_SR_2(2 downto 0);
			A_RCV_SR_3(3 downto 1) <= A_RCV_SR_3(2 downto 0);
			A_RCV_SR_4(3 downto 1) <= A_RCV_SR_4(2 downto 0);
			                        
		end if;              
	end process;
	
-------------------------------------------------------------------------
	process (clk, RD_lower_sr, A_RCV_SR_0, A_RCV_SR_1, A_RCV_SR_2, 
			 A_RCV_SR_3, A_RCV_SR_4)
	begin
		if rising_edge(clk) then
			if (RD_lower_sr = '1') then
				Parallel_A_in(3 downto 0)   <= A_RCV_SR_0(3 downto 0);
				Parallel_A_in(7 downto 4)   <= A_RCV_SR_1(3 downto 0);
				Parallel_A_in(11 downto 8)  <= A_RCV_SR_2(3 downto 0);
				Parallel_A_in(15 downto 12) <= A_RCV_SR_3(3 downto 0);
				Parallel_A_in(19 downto 16) <= A_RCV_SR_4(3 downto 0);
			end if;                           
		end if;		                   
	end process;	

	process (clk, nreset)
	begin
		if(nreset = '0')  then 
			WR_mem <= '0';
		elsif rising_edge(clk) then
			WR_mem <= WR_lower_mem;
		end if;
	end process;

end;