Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

-------------------------------------------------------------------------------
-- MODULE DESCRIPTION, March 26, 2010

entity LOWER_BUS_CTRL_MUX is 
	port( 
			SRAM_LD			: in std_logic_vector(15 downto 0);
			SRAM_LA			: in std_logic_vector(17 downto 0);
			SRAM_WE_n		: in std_logic;
			SRAM_CS_n		: in std_logic;
			SRAM_OE_n		: in std_logic;
			SRAM_ENA		: in std_logic;
			
			VME_LD			: in std_logic_vector(31 downto 16);
			VME_ACTIVE_n    : in std_logic;
			VME_WRITE_nREAD	: in std_logic;
			
			LD_OUT			: out std_logic_vector(15 downto 0);
			LA_OUT			: out std_logic_vector(17 downto 0);
			Mem_WE_n		: out std_logic;
			Mem_CS_n		: out std_logic;
			Mem_OE_n		: out std_logic;
			
			Drive_Data_Out  : out std_logic;
			Drive_Addr_Out	: out std_logic; --J2C_Active_n 
			
			J2C_Active_n	: out std_logic;
			J2C_AB_nBA	 	: out std_logic  --Always High
			
		);
	end LOWER_BUS_CTRL_MUX;
	
architecture main of LOWER_BUS_CTRL_MUX is	
begin
	process (SRAM_ENA, SRAM_LD, SRAM_LA, SRAM_WE_n, SRAM_CS_n, SRAM_OE_n, 
			 VME_LD, VME_ACTIVE_n, VME_WRITE_nREAD)
	begin
		J2C_AB_nBA	 <= '1'; -- directs VME address lines from VME bus to board
		
		if  (SRAM_ENA = '1') then
			LD_OUT   <= SRAM_LD;
			LA_OUT   <= SRAM_LA;
			Mem_WE_n <= SRAM_WE_n;	
			Mem_CS_n <= SRAM_CS_n;	
			Mem_OE_n <= SRAM_OE_n;	
			
			Drive_Data_Out  <= NOT (NOT SRAM_OE_n);
			Drive_Addr_Out	<= '1'; -- address lines driven out to SRAM
			J2C_Active_n	<= '1'; 

		else
			LD_OUT   <= VME_LD;
			LA_OUT   <= (others => '1'); -- not driven, don't care
			Mem_WE_n <= '1'; -- disabled	
			Mem_CS_n <= '1'; -- disabled
			Mem_OE_n <= '1'; -- disabled	
			
			Drive_Data_Out  <= (NOT VME_ACTIVE_n) AND (NOT VME_WRITE_nREAD);
			Drive_Addr_Out	<= '0';			
			J2C_Active_n	<= '0'; -- address lines are driven by VME bus
		end if;
			
	end process;
end;