Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

-------------------------------------------------------------------------------
-- MODULE DESCRIPTION, April 6, 2016
--
-- This is where the address bits are decoded for the Lower FPGA memory map.
 


entity Lower_Addr_Decoder_Upper_Bus_v2 is 
	port( 
			Addr_to_Mem	: in std_logic_vector(23 downto 0);
			WR_Pulse		: in std_logic;
				
			FC_Data		: in std_logic_vector(31 downto 0);			
			BC_Data		: in std_logic_vector(31 downto 0);
			INJ_Data		: in std_logic_vector(31 downto 0);
			PARPH_Data	: in std_logic_vector(31 downto 0);
			PARAM_Data	: in std_logic_vector(31 downto 0);
			CTRL_Data	: in std_logic_vector(31 downto 0);
			IO_Data		: in std_logic_vector(31 downto 0);
			SRAM_Data   : in std_logic_vector(31 downto 0);
			
			Data_Out		: out std_logic_vector(31 downto 0);		
			
			WR_FC			: out std_logic;		
			WR_BC			: out std_logic;		
			WR_INJ		: out std_logic;			
			WR_PPH		: out std_logic;			
			WR_SRAM		: out std_logic	
		);
	end Lower_Addr_Decoder_Upper_Bus_v2;
	
architecture main of Lower_Addr_Decoder_Upper_Bus_v2 is	

signal WR   : std_logic;
signal ADDR : std_logic_vector(23 downto 0);

signal WR_INJ1 : std_logic;
signal WR_INJ2 : std_logic;

begin
	
	WR <= WR_Pulse;
	ADDR <= Addr_to_Mem;
	
	process (ADDR, WR, FC_Data, BC_Data, INJ_Data,
			 PARPH_Data, PARAM_Data, CTRL_Data, IO_Data, SRAM_Data)
	begin
		WR_FC	<= '0';
		WR_BC	<= '0';
		WR_INJ1	<= '0';
		WR_INJ2	<= '0';
		WR_PPH	<= '0';
 		WR_SRAM <= '0';
 		Data_Out <= (others => '0');
             
		if   (ADDR < FREQUENCY_CURVE1    ) then  NULL; -- Upper FPGA Memory Space
		elsif(ADDR < FREQUENCY_CURVE1 + X"1000") then  -- X"1000" bytes is 1024, 4 byte values.
												 WR_FC   <= WR;	Data_Out <= FC_Data;
												 WR_INJ1 <= WR; -- injection portion of FC
		elsif(ADDR < BIAS_CURVE          ) then  WR_FC   <= WR;	Data_Out <= FC_Data;
		elsif(ADDR < X"032000"		      ) then  WR_BC   <= WR;	Data_Out <= BC_Data;
		elsif(ADDR < X"03E000"			    ) then NULL;
		elsif(ADDR < X"03F000"			    ) then  WR_INJ2 <= WR;	Data_Out <= INJ_Data;
		elsif(ADDR < PARAM_LOWER_FIRST   ) then  WR_PPH  <= WR;	Data_Out <= PARPH_Data;
		elsif(ADDR < VME_CTRL_LOWER_FIRST) then						Data_Out <= PARAM_Data;
		elsif(ADDR < IO_REGISTER_SPACE   ) then						Data_Out <= CTRL_Data;
		elsif(ADDR < SRAM_MEMORY         ) then   					Data_Out <= IO_Data;
		elsif(ADDR < SRAM_MEMORY_LAST    ) then  WR_SRAM <= WR;	Data_Out <= SRAM_Data ;		
		else						    		 NULL;                     
		end if;
	end process;
	
	WR_INJ <= WR_INJ1 OR WR_INJ2;
end;