Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

----------------------------------------------------------------
-- Placement of these 32 bit registers is controlled by the 
-- constant VME_CTRL_LOWER_FIRST.  This constant is defined in
-- the included file DDS_VME_CONSTANTS.vhd.

entity Lower_VME_Ctrl_Regs is 
	port( 	
			clk			: in std_logic;
			nreset		: in std_logic;
			
			Data_to_Mem : in std_logic_vector(31 downto 0);
			Addr_to_Mem	: in std_logic_vector(23 downto 0);
			WR			: in std_logic;	
			
			reg_Data_Out	: out std_logic_vector(31 downto 0);
			reg_0		: out std_logic_vector(31 downto 0);
			reg_1		: out std_logic_vector(31 downto 0);
			reg_2		: out std_logic_vector(31 downto 0);
			reg_3		: out std_logic_vector(31 downto 0);
			reg_4		: out std_logic_vector(31 downto 0);
			reg_5		: out std_logic_vector(31 downto 0);
			reg_6		: out std_logic_vector(31 downto 0);
			reg_7		: out std_logic_vector(31 downto 0)			        
		);
	end Lower_VME_Ctrl_Regs;
	
architecture main of Lower_VME_Ctrl_Regs is	

signal ADDR 		: std_logic_vector(23 downto 0);

signal reg_0_out	: std_logic_vector(31 downto 0);
signal reg_1_out	: std_logic_vector(31 downto 0);
signal reg_2_out	: std_logic_vector(31 downto 0);
signal reg_3_out	: std_logic_vector(31 downto 0);
signal reg_4_out	: std_logic_vector(31 downto 0);
signal reg_5_out	: std_logic_vector(31 downto 0);
signal reg_6_out	: std_logic_vector(31 downto 0);
signal reg_7_out	: std_logic_vector(31 downto 0);
            
begin
	
	ADDR <= Addr_to_Mem;
	
	PARAMETER_REGISTERS: process(clk, nreset, WR, ADDR)
	begin	
		if rising_edge(clk) then
			if (nreset = '0') then
				reg_0_out <= X"000F0000";--(others => '0');
				reg_1_out <= X"80008000";--(others => '0');
				reg_2_out <= (others => '0');
				reg_3_out <= (others => '0');
				reg_4_out <= (others => '0');
				reg_5_out <= (others => '0');
				reg_6_out <= (others => '0');
				reg_7_out <= (others => '0');
			elsif(WR = '1') then
				if   (ADDR = VME_CTRL_LOWER_FIRST     ) then reg_0_out <= Data_to_Mem;
				elsif(ADDR = VME_CTRL_LOWER_FIRST + 4 ) then reg_1_out <= Data_to_Mem;
				elsif(ADDR = VME_CTRL_LOWER_FIRST + 8 ) then reg_2_out <= Data_to_Mem;
				elsif(ADDR = VME_CTRL_LOWER_FIRST + 12) then reg_3_out <= Data_to_Mem;
				elsif(ADDR = VME_CTRL_LOWER_FIRST + 16) then reg_4_out <= Data_to_Mem;
				elsif(ADDR = VME_CTRL_LOWER_FIRST + 20) then reg_5_out <= Data_to_Mem;
				elsif(ADDR = VME_CTRL_LOWER_FIRST + 24) then reg_6_out <= Data_to_Mem;
				elsif(ADDR = VME_CTRL_LOWER_FIRST + 28) then reg_7_out <= Data_to_Mem;
				end if;           
			end if;
		end if;
	end process;
       
	DATA_OUT_MUX: 
	process(ADDR, reg_0_out, reg_1_out, reg_2_out, reg_3_out, 
					reg_4_out, reg_5_out, reg_6_out, reg_7_out)
	begin
		if   (ADDR = VME_CTRL_LOWER_FIRST     ) then reg_Data_Out <= reg_0_out;
		elsif(ADDR = VME_CTRL_LOWER_FIRST + 4 ) then reg_Data_Out <= reg_1_out;
		elsif(ADDR = VME_CTRL_LOWER_FIRST + 8 ) then reg_Data_Out <= reg_2_out;
		elsif(ADDR = VME_CTRL_LOWER_FIRST + 12) then reg_Data_Out <= reg_3_out;
		elsif(ADDR = VME_CTRL_LOWER_FIRST + 16) then reg_Data_Out <= reg_4_out;
		elsif(ADDR = VME_CTRL_LOWER_FIRST + 20) then reg_Data_Out <= reg_5_out;
		elsif(ADDR = VME_CTRL_LOWER_FIRST + 24) then reg_Data_Out <= reg_6_out;
		elsif(ADDR = VME_CTRL_LOWER_FIRST + 28) then reg_Data_Out <= reg_7_out;
		else		 	 			  			     reg_Data_Out <= (others => '0');
		end if;                                 
				
	end process;

	reg_0 <= reg_0_out;
	reg_1 <= reg_1_out;
	reg_2 <= reg_2_out;
	reg_3 <= reg_3_out;
	reg_4 <= reg_4_out;
	reg_5 <= reg_5_out;
	reg_6 <= reg_6_out;
	reg_7 <= reg_7_out;
          
end;