Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

------------------------------------------------------

entity MCP4021_Interface2 is
	port(
			clk			: in std_logic;
			reset_n		: in std_logic;
			DPOT_Addr	: in std_logic_vector(4 downto 0);
			Direction	: in std_logic;
			Step		: in std_logic;
			Nsteps		: in std_logic_vector(5 downto 0);
			Ena_Store	: in std_logic;
			Ena_DPOT		: in std_logic;
			
			TEST_ADJ_CSn : out std_logic;
			TEST_ADJ_UDn : out std_logic;
			
			INP_OFFS_CS1n : out std_logic;
			INP_OFFS_UD1n : out std_logic;
			INP_GAIN_CS1n : out std_logic;
			INP_GAIN_UD1n : out std_logic;
			INP_OFFS_CS2n : out std_logic;
			INP_OFFS_UD2n : out std_logic;
			INP_GAIN_CS2n : out std_logic;
			INP_GAIN_UD2n : out std_logic;
			INP_OFFS_CS3n : out std_logic;
			INP_OFFS_UD3n : out std_logic;
			INP_GAIN_CS3n : out std_logic;
			INP_GAIN_UD3n : out std_logic;
			INP_OFFS_CS4n : out std_logic;
			INP_OFFS_UD4n : out std_logic;
			INP_GAIN_CS4n : out std_logic;
			INP_GAIN_UD4n : out std_logic;
               
			DAC_OFFS_CS1n : out std_logic;
			DAC_OFFS_UD1n : out std_logic;
			DAC_GAIN_CS1n : out std_logic;
			DAC_GAIN_UD1n : out std_logic;
			DAC_OFFS_CS2n : out std_logic;
			DAC_OFFS_UD2n : out std_logic;
			DAC_GAIN_CS2n : out std_logic;
			DAC_GAIN_UD2n : out std_logic;
			DAC_OFFS_CS3n : out std_logic;
			DAC_OFFS_UD3n : out std_logic;
			DAC_GAIN_CS3n : out std_logic;
			DAC_GAIN_UD3n : out std_logic;
			DAC_OFFS_CS4n : out std_logic;
			DAC_OFFS_UD4n : out std_logic;
			DAC_GAIN_CS4n : out std_logic;
			DAC_GAIN_UD4n : out std_logic
			    
		);
end MCP4021_Interface2;

architecture main of MCP4021_Interface2 is

-- Signals to latch and syncronize the inputs ----------------------------
signal led			: std_logic_vector(2 downto 0);
signal step_cmd	: std_logic;

signal ltch_dir		: std_logic;
signal ltch_step	: std_logic;

signal sync_dir		: std_logic_vector(1 downto 0);
signal sync_step	: std_logic_vector(1 downto 0);

signal dir			: std_logic;
signal go			: std_logic;
signal clr_cmd		: std_logic;

-- State Machine Outputs -------------------------------------------------
signal cs_strobe	: std_logic;
signal step_pulse	: std_logic;

-- Counter to implement chip select low to U_nD rising edge timing -------
signal count		: std_logic_vector(1 downto 0);
signal cnt_clrn		: std_logic;
signal cnt_done		: std_logic;

-- Clock Divider ---------------------------------------------------------
signal clkdiv		: std_logic_vector(4 downto 0);
signal clk_640ns	: std_logic;

-- Step Counter ----------------------------------------------------------
signal stepcnt		: std_logic_vector(5 downto 0);
signal stepcnt_clrn	: std_logic;
signal stepcnt_load : std_logic;
signal stepcnt_incr	: std_logic;
signal stepcnt_done : std_logic;

-- STATE_MACHINE --------------------------------------
type STATE_TYPE is
	(idle, setup_dir, hold_dir, setup_step, do_step, setup_delay, wr_wiper);
	signal CS, NS : STATE_TYPE;	

--========================================================
--                  B E G I N
--========================================================
begin

TRIGGER_LED: -- Detect leading edge of Step. Step pulse might be long.
process(clk, reset_n, Step, Ena_DPOT)
begin
	if   (reset_n = '0') 	then led <= "000";
	elsif rising_edge(clk)	then
		led(0) <= Step AND Ena_DPOT;
		led(1) <= led(0);
		led(2) <= led(1);
	end if;
end process;

step_cmd <= led(1) and not led(2);

--------------------------------------------------------------------------------
LATCH_DIRECTION: -- Latch Direction at the step_pulse.
process(clk, reset_n, step_cmd)
begin
	if (reset_n = '0') 	   		then ltch_dir  <= '0';
	elsif rising_edge(clk) 		then
		if   (step_cmd = '1')	then ltch_dir <= Direction;
		end if;	
	end if;
end process;

dir <= ltch_dir;
--------------------------------------------------------------------------------
HOLD_STEP_PULSE: -- Hold the step pulse to syncronize to the 640ns Clock.
process(clk, reset_n, clr_cmd, step_cmd)
begin
	if (reset_n = '0') 			then ltch_step <= '0';
	elsif rising_edge(clk) then
		if   (step_cmd  = '1')	then ltch_step <= '1';			
		elsif(clr_cmd = '1')	then ltch_step <= '0';			
		end if;	
	end if;
end process;

---------------------------------------------------------------------------------
SYNC_TO_640ns_CLK: -- Produce step pulse syncd to the 640ns clock.
process(clk, clk_640ns, reset_n, ltch_step)
begin
	if (reset_n = '0') 				then sync_step <= "00";			
	elsif (rising_edge(clk_640ns)) 	then
		sync_step(0) <= ltch_step;	
		sync_step(1) <= sync_step(0);
	end if;
end process;

go 		<= sync_step(0) AND NOT sync_step(1);
clr_cmd 	<= sync_step(1);

---------------------------------------------------------------------------------
CLOCK_DIVIDER: process(clk, reset_n)
begin
		if(reset_n = '0') then clkdiv <= "00000";
		elsif(rising_edge(clk)) then 
			clkdiv <= clkdiv + 1;
			clk_640ns <= clkdiv(4);
		end if;
end process;

---------------------------------------------------------------------------------
SETUP_DELAY_TIMER:
process(clk_640ns, reset_n, cnt_clrn)
begin
	if (reset_n = '0') OR (cnt_clrn = '0') then count <= "00";
	elsif(rising_edge(clk_640ns)) then 
		count <= count + 1;
		if(count = "11") then 	cnt_done <= '1';
		else					cnt_done <= '0';
		end if;
	end if;
end process;

---------------------------------------------------------------------------------
STEP_COUNTER: counter -- Count number of dpot steps taken
generic map(6)
port map(	clk => clk_640ns,
			clrn => stepcnt_clrn,
			ena  => stepcnt_incr,
			load => '0',
			updn => '1',
			ld_data  => (others => '0'),
			flag_cnt => Nsteps,
			flag_out => stepcnt_done,
			cnt_out  => open 
		);

--=======================================================================
-- STATE MACHINE  
--=======================================================================
SM_REG: 
process(clk_640ns, reset_n)
begin
	if(reset_n = '0')				then CS <= idle;
	elsif(rising_edge(clk_640ns))	then CS <= NS;
	end if;
end process;

SM_NEXT_STATE: 
--(idle, setup_dir, hold_dir, setup_step, do_step, setup_delay, wr_wiper);
process(CS, go, dir, cnt_done, stepcnt_done, Ena_Store)
begin
	-- DEFAULT VALUES ----------------
	cs_strobe	 <= '1';
	step_pulse	 <= '0';
	cnt_clrn	 <= '0';
	stepcnt_incr <= '0';
	stepcnt_clrn <= '1';
	
	case CS is
		when idle => 
			if(go = '1')	then			NS <= setup_dir;
			else							NS <= idle;
			end if;

			stepcnt_clrn <= '0';
			-------------------------------------------
			cs_strobe	<= '1'; 	step_pulse	<= dir;
			-------------------------------------------
		
		when setup_dir =>					NS <= hold_dir;
			-------------------------------------------
			cs_strobe	<= '1';		step_pulse	<= dir;
			-------------------------------------------
		
		when hold_dir =>					NS <= setup_step;
			-------------------------------------------
			cs_strobe	<= '0';		step_pulse	<= dir;
			-------------------------------------------
		
		when setup_step =>
			cnt_clrn	<= '1';
			if(cnt_done = '1') then 		NS <= do_step;
			else							NS <= setup_step;
			end if;
			--------------------------------------------
			cs_strobe	<= '0';	 	step_pulse	<= '0';
			--------------------------------------------
	                          
		when do_step =>						NS <= setup_delay;
			stepcnt_incr <= '1';
			-------------------------------------------
			cs_strobe	<= '0';		step_pulse	<= '1';
			-------------------------------------------
		
		when setup_delay =>			
			if   (stepcnt_done = '1') then 	NS <= wr_wiper;
				step_pulse  <= dir XOR Ena_Store;
			else							NS <= do_step;
				step_pulse	<= '0';
			end if;
			--------------------
			cs_strobe	<= '0';	
			--------------------
		
		when wr_wiper =>					NS <= idle;
			-------------------------------------------
			cs_strobe	<= '1';						
			step_pulse  <= dir XOR Ena_Store;
			-------------------------------------------
			
		when others =>              		NS <= idle;
	end case;
end process;

-- OUTPUTS --------------------------------------------------
process(DPOT_Addr, cs_strobe)
begin
	-- DEFAULT VALUES ----------------
	TEST_ADJ_CSn  <= '1';
	INP_OFFS_CS1n <= '1';
	INP_OFFS_CS2n <= '1';
	INP_OFFS_CS3n <= '1';
	INP_OFFS_CS4n <= '1';
	DAC_OFFS_CS1n <= '1';
	DAC_OFFS_CS2n <= '1';
	DAC_OFFS_CS3n <= '1';
	DAC_OFFS_CS4n <= '1';

	INP_GAIN_CS1n <= '1';
	INP_GAIN_CS2n <= '1';
	INP_GAIN_CS3n <= '1';
	INP_GAIN_CS4n <= '1';
	DAC_GAIN_CS1n <= '1';
	DAC_GAIN_CS2n <= '1';
	DAC_GAIN_CS3n <= '1';
	DAC_GAIN_CS4n <= '1';
        
				
	case DPOT_Addr is		
		when "00000" => 	INP_OFFS_CS1n <= cs_strobe;
		when "00001" => 	INP_GAIN_CS1n <= cs_strobe;
		
		when "00010" => 	INP_OFFS_CS2n <= cs_strobe;
		when "00011" => 	INP_GAIN_CS2n <= cs_strobe;
		
		when "00100" => 	INP_OFFS_CS3n <= cs_strobe;
		when "00101" => 	INP_GAIN_CS3n <= cs_strobe;
		
		when "00110" => 	INP_OFFS_CS4n <= cs_strobe;
		when "00111" => 	INP_GAIN_CS4n <= cs_strobe;
		
		when "01000" => 	DAC_OFFS_CS1n <= cs_strobe;
		when "01001" => 	DAC_GAIN_CS1n <= cs_strobe;
		
		when "01010" => 	DAC_OFFS_CS2n <= cs_strobe;
		when "01011" => 	DAC_GAIN_CS2n <= cs_strobe;
		
		when "01100" => 	DAC_OFFS_CS3n <= cs_strobe;
		when "01101" => 	DAC_GAIN_CS3n <= cs_strobe;
		
		when "01110" => 	DAC_OFFS_CS4n <= cs_strobe;
		when "01111" => 	DAC_GAIN_CS4n <= cs_strobe;
		
		when "10000" => 	TEST_ADJ_CSn  <= cs_strobe;
		
		when others =>  NULL;
	end case;
end process;
                                         
	TEST_ADJ_UDn  <=	step_pulse;	
		
	INP_GAIN_UD1n <=	step_pulse;
	INP_GAIN_UD2n <=	step_pulse;
	INP_GAIN_UD3n <=	step_pulse;
	INP_GAIN_UD4n <=	step_pulse;
	DAC_GAIN_UD1n <=	step_pulse;
	DAC_GAIN_UD2n <=	step_pulse;
	DAC_GAIN_UD3n <=	step_pulse;
	DAC_GAIN_UD4n <=	step_pulse;
	
	INP_OFFS_UD1n <=	step_pulse;
	INP_OFFS_UD2n <=	step_pulse;
	INP_OFFS_UD3n <=	step_pulse;
	INP_OFFS_UD4n <=	step_pulse;
	DAC_OFFS_UD1n <=	step_pulse;
	DAC_OFFS_UD2n <=	step_pulse;
	DAC_OFFS_UD3n <=	step_pulse;
	DAC_OFFS_UD4n <=	step_pulse;

end;----------------------------------------------------------