Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

-------------------------------------------------------------------------------
-- MODULE DESCRIPTION, March 17, 2010
--
-- This is where the address bits are decoded for the Lower FPGA memory map.
-- If 


entity Parameter_Registers is 
	port( 	
			clk			: in std_logic;
			nreset		: in std_logic;
			
			Data_to_Mem : in std_logic_vector(31 downto 0);
			Addr_to_Mem	: in std_logic_vector(25 downto 0);
			WR			: in std_logic;	
			
			Param_Data_Out	: out std_logic_vector(31 downto 0);
			Param_0		: out std_logic_vector(31 downto 0);
			Param_1		: out std_logic_vector(31 downto 0);
			Param_2		: out std_logic_vector(31 downto 0);
			Param_3		: out std_logic_vector(31 downto 0);
			Param_4		: out std_logic_vector(31 downto 0);
			Param_5		: out std_logic_vector(31 downto 0);
			Param_6		: out std_logic_vector(31 downto 0);
			Param_7		: out std_logic_vector(31 downto 0);
			
			Run_Curves_Sync	: out std_logic;
			updated_0   	: out std_logic;
			updated_1		: out std_logic		        
		);
	end Parameter_Registers;
	
architecture main of Parameter_Registers is	

signal ADDR 		: std_logic_vector(25 downto 0);

signal param_0_out	: std_logic_vector(31 downto 0);
signal param_1_out	: std_logic_vector(31 downto 0);
signal param_2_out	: std_logic_vector(31 downto 0);
signal param_3_out	: std_logic_vector(31 downto 0);
signal param_4_out	: std_logic_vector(31 downto 0);
signal param_5_out	: std_logic_vector(31 downto 0);
signal param_6_out	: std_logic_vector(31 downto 0);
signal param_7_out	: std_logic_vector(31 downto 0);
            
begin
	
	ADDR <= Addr_to_Mem;
	
	PARAMETER_REGISTERS: process(clk, nreset, WR, ADDR)
	begin	
		if rising_edge(clk) then
			if (nreset = '0') then
				param_0_out <= (others => '0');
				param_1_out <= (others => '0');
				param_2_out <= (others => '0');
				param_3_out <= (others => '0');
				param_4_out <= (others => '0');
				param_5_out <= (others => '0');
				param_6_out <= (others => '0');
				param_7_out <= (others => '0');
			elsif(WR = '1') then
				if   (ADDR = PARAM_LOWER_FIRST     ) then param_0_out <= Data_to_Mem;
				elsif(ADDR = PARAM_LOWER_FIRST + 4 ) then param_1_out <= Data_to_Mem;
				elsif(ADDR = PARAM_LOWER_FIRST + 8 ) then param_2_out <= Data_to_Mem;
				elsif(ADDR = PARAM_LOWER_FIRST + 12) then param_3_out <= Data_to_Mem;
				elsif(ADDR = PARAM_LOWER_FIRST + 16) then param_4_out <= Data_to_Mem;
				elsif(ADDR = PARAM_LOWER_FIRST + 20) then param_5_out <= Data_to_Mem;
				elsif(ADDR = PARAM_LOWER_FIRST + 24) then param_6_out <= Data_to_Mem;
				elsif(ADDR = PARAM_LOWER_FIRST + 28) then param_7_out <= Data_to_Mem;
				
				--NOTE BELOW -- (ADDR = PARAM_LOWER_FIRST + 32) then Run_Curves_Sync <= WR;
				
				end if;
			end if;
		end if;
	end process;
       
	DATA_OUT_MUX: 
	process(ADDR, param_0_out, param_1_out, param_2_out, param_3_out, 
					param_4_out, param_5_out, param_6_out, param_7_out)
	begin
		if   (ADDR = PARAM_LOWER_FIRST     ) then Param_Data_Out <= param_0_out;
		elsif(ADDR = PARAM_LOWER_FIRST + 4 ) then Param_Data_Out <= param_1_out;
		elsif(ADDR = PARAM_LOWER_FIRST + 8 ) then Param_Data_Out <= param_2_out;
		elsif(ADDR = PARAM_LOWER_FIRST + 12) then Param_Data_Out <= param_3_out;
		elsif(ADDR = PARAM_LOWER_FIRST + 16) then Param_Data_Out <= param_4_out;
		elsif(ADDR = PARAM_LOWER_FIRST + 20) then Param_Data_Out <= param_5_out;
		elsif(ADDR = PARAM_LOWER_FIRST + 24) then Param_Data_Out <= param_6_out;
		elsif(ADDR = PARAM_LOWER_FIRST + 28) then Param_Data_Out <= param_7_out;
		else		 				  Param_Data_Out <= (others => '0');
		end if;                                 
				
	end process;

	Param_0 <= param_0_out;
	Param_1 <= param_1_out;
	Param_2 <= param_2_out;
	Param_3 <= param_3_out;
	Param_4 <= param_4_out;
	Param_5 <= param_5_out;
	Param_6 <= param_6_out;
	Param_7 <= param_7_out;
	
	UPDATED_PARAM_INDICATION:
	process(ADDR, WR)
    begin
		updated_0 <= '0';
		updated_1 <= '0';
		Run_Curves_Sync <= '0';
		
		if   (ADDR = PARAM_LOWER_FIRST     ) then updated_0 <= WR;
		elsif(ADDR = PARAM_LOWER_FIRST + 4 ) then updated_1 <= WR;
		elsif(ADDR = PARAM_LOWER_FIRST + 32) then Run_Curves_Sync <= WR;
		end if;                                 
    end process;
          
end;