Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

------------------------------------------------------
-- MODULE DESCRIPTION
-- 
------------------------------------------------------

entity AD9910_Control_Mux_v2 is 
	port( 
		SEL_Setup_nUpdate	: in std_logic;

		Setup_TxData	: in std_logic_vector(71 downto 0); -- data to transmit
		Setup_TxStrb	: in std_logic; -- strobe to latch data to transmit
		Setup_NBits		: in std_logic_vector(6 downto 0);
		Setup_MSB_nLSB	: in std_logic; -- send MSB or LSB first indication	
		Setup_Ena_IOReset	: in std_logic;
		Setup_CS_1		: in std_logic;
		Setup_CS_2		: in std_logic;
		Setup_CS_3		: in std_logic;
		Setup_CS_4		: in std_logic;
                
		Update_TxData_1	: in std_logic_vector(71 downto 0); -- data to transmit
		Update_TxData_2	: in std_logic_vector(71 downto 0); -- data to transmit
		Update_TxData_3	: in std_logic_vector(71 downto 0); -- data to transmit
		Update_TxData_4	: in std_logic_vector(71 downto 0); -- data to transmit
		Update_TxStrb	: in std_logic; -- strobe to latch data to transmit
		Update_NBits	: in std_logic_vector(6 downto 0);
		Update_MSB_nLSB	: in std_logic; -- send MSB or LSB first indication	
		Update_Ena_IOReset	: in std_logic;
		Update_CS_1		: in std_logic;
		Update_CS_2		: in std_logic;
		Update_CS_3		: in std_logic;
		Update_CS_4		: in std_logic;
		         
		TxStrb		: out std_logic; -- strobe to latch data to transmit
		TxData_1	: out std_logic_vector(71 downto 0); -- data to transmit
		TxData_2	: out std_logic_vector(71 downto 0); -- data to transmit
		TxData_3	: out std_logic_vector(71 downto 0); -- data to transmit
		TxData_4	: out std_logic_vector(71 downto 0); -- data to transmit
		BitCount	: out std_logic_vector(6 downto 0);
		MSB_nLSB    : out std_logic; -- send MSB or LSB first indication	
		Ena_IOReset : out std_logic;
		DDS_CS_1	: out std_logic;
		DDS_CS_2	: out std_logic;
		DDS_CS_3	: out std_logic;
		DDS_CS_4	: out std_logic
		);
	end AD9910_Control_Mux_v2 ;
	
architecture main of AD9910_Control_Mux_v2  is

begin
	process (SEL_Setup_nUpdate,
			Setup_TxData,		
			Setup_TxStrb,		
			Setup_NBits,		
			Setup_MSB_nLSB,		
			Setup_Ena_IOReset,		
			Setup_CS_1,			
			Setup_CS_2,			
			Setup_CS_3,			
			Setup_CS_4,			
			Update_TxData_1,		
			Update_TxData_2,		
			Update_TxData_3,		
			Update_TxData_4,		
			Update_TxStrb,		
			Update_NBits,		
			Update_MSB_nLSB,		
			Update_Ena_IOReset,		
			Update_CS_1,
			Update_CS_2,
			Update_CS_3,
			Update_CS_4
			         )
	begin
		if(SEL_Setup_nUpdate = '1') then
			TxData_1	<= Setup_TxData;	
			TxData_2	<= Setup_TxData;	
			TxData_3	<= Setup_TxData;	
			TxData_4	<= Setup_TxData;
				
			TxStrb		<= Setup_TxStrb;	
			BitCount	<= Setup_NBits;	
			MSB_nLSB    <= Setup_MSB_nLSB;	
			Ena_IOReset <= Setup_Ena_IOReset;	
			DDS_CS_1	<= Setup_CS_1;		
			DDS_CS_2	<= Setup_CS_2;		
			DDS_CS_3	<= Setup_CS_3;		
			DDS_CS_4	<= Setup_CS_4;		
		else	                   
			TxData_1	<= Update_TxData_1;				               
			TxData_2	<= Update_TxData_2;				               
			TxData_3	<= Update_TxData_3;				               
			TxData_4	<= Update_TxData_4;	
						               
			TxStrb		<= Update_TxStrb;				             
			BitCount	<= Update_NBits;		
			MSB_nLSB    <= Update_MSB_nLSB;				
			Ena_IOReset <= Update_Ena_IOReset;				
			DDS_CS_1	<= Update_CS_1;		
			DDS_CS_2	<= Update_CS_2;		
			DDS_CS_3	<= Update_CS_3;		
			DDS_CS_4	<= Update_CS_4;		
		end if;	                   
	end process;           
end main;		
		
			