Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

----------------------------------------------------------------

entity DAC3_Data_Select is 
	port( 	
			clk					: in std_logic;
			nreset				: in std_logic;
			Select_Data			: in std_logic_vector( 3 downto 0);
			DDS_Update_Strobe	: in std_logic;
						
			Paraphase_Sum		: in std_logic_vector(15 downto 0);		
			Paraphase_Crv		: in std_logic_vector(15 downto 0);
			Phase_Offset_A		: in std_logic_vector(15 downto 0);
			Phase_Offset_B		: in std_logic_vector(15 downto 0);
			Phase_A				: in std_logic_vector(15 downto 0);						
			Phase_B				: in std_logic_vector(15 downto 0);						                               
			Ref_Phase			: in std_logic_vector(15 downto 0);
			Rpos_Phase		 	: in std_logic_vector(15 downto 0);
						
			data_out	 			: out std_logic_vector(15 downto 0)
			
		);
	end DAC3_Data_Select;
	
architecture main of DAC3_Data_Select is	

signal r0,r1,r2,r3,r4,r5,r6,r7	: std_logic_vector(15 downto 0);


            
--WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW
--MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
begin
------------------------------------------------------------------------
DAC_DATA_REGISTERS: 
process(clk, nreset, DDS_Update_Strobe)
begin
	if   (nreset = '0') then 
										r0 <= (others => '0');
										r1 <= (others => '0');
										r2 <= (others => '0');
										r3 <= (others => '0');
										r4 <= (others => '0');
										r5 <= (others => '0');
										r6 <= (others => '0');
										r7 <= (others => '0');
	elsif rising_edge(clk) then
		if   (DDS_Update_Strobe = '1') then
										r0(15) 				<= not Paraphase_Sum(15);
										r0(14 downto 0)	<= Paraphase_Sum(14 downto 0);

										r1(15) 				<= not Paraphase_Crv(14);
										r1(14 downto 1) 	<= Paraphase_Crv(13 downto 0);
										r1(0) 				<= '0';

										r2(15) 				<= not Phase_Offset_A(15);
										r2(14 downto 0)	<= Phase_Offset_A(14 downto 0);

										r3(15) 				<= not Phase_Offset_B(15);
										r3(14 downto 0)	<= Phase_Offset_B(14 downto 0);
										
										r4(15)				<= not Phase_A(15);
										r4(14 downto 0)	<= Phase_A(14 downto 0);
										
										r5(15)				<= not Phase_B(15);
										r5(14 downto 0)	<= Phase_B(14 downto 0);
										
										r6(15)				<= not Ref_Phase(15);
										r6(14 downto 0)	<= Ref_Phase(14 downto 0);
										
										r7(15) 				<= not Rpos_Phase(14);
										r7(14 downto 1) 	<= Rpos_Phase(13 downto 0);
										r7(0) 				<= '0';
		end if;
	end if;
end process;

-------------------------------------------------------------------------
DATA_SELECT:
process( Select_Data, r0,r1,r2,r3,r4,r5,r6,r7)
begin
	case Select_Data is
		when X"0" =>	data_out   <= r0;
		when X"1" =>	data_out   <= r1;
		when X"2" =>	data_out   <= r2;
		when X"3" =>	data_out   <= r3;
		when X"4" =>	data_out   <= r4;
		when X"5" =>	data_out   <= r5;
		when X"6" =>	data_out   <= r6;
		when X"7" =>	data_out   <= r7;
		when others =>	data_out   <= r0;
	end case;
end process;

end;