Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

-------------------------------------------------------------------------------
-- MODULE DESCRIPTION, 

entity DDS_App_Control_Mux is 
	port( 
			VME_control_ena		: in std_logic;
			VME_Ctrl					: in std_logic_vector(15 downto 0);
			Flash_Bus_Req			: in std_logic;
			
			Verify_All				: in std_logic; -- Forces update of all Parameters
			Set_CTBFREE_Sem			: in std_logic; -- Initialize the CBTFREE SEM
			Preset_Curve_Status 	: in std_logic; -- Forces all curves to be setup from buffers
			Reset_Run_Curves		: in std_logic; -- Initializes Run Mode DDS Curves
			
			Update_Trigger			: in std_logic; -- Trigger the Parameter Update Process
			Curve_Req_Trigger		: in std_logic; -- Trigger the Curve Request/Update Process
			Curve_Intrp_Trigger 	: in std_logic; -- Trigger the Curve Interpolation Process
			Bootup_Trigger			: in std_logic; -- Trigger the Flash Restore Process
			Backup_Trigger			: in std_logic; -- Trigger the Flash Store Process
			Flash_Bus_Permit		: in std_logic; -- Signal Flash to continue the Store Process
			DDS_Ready				: in std_logic;

			Run_Curves				: in std_logic;
         DDS_1us_Update_Pulse	: in std_logic;
         Rewind_Curves			: in std_logic;           
						
			Verify_All_x				: out std_logic; -- Forces update of all Parameters
			Set_CTBFREE_Sem_x			: out std_logic; -- Initialize the CBTFREE SEM
			Preset_Curve_Status_x 	: out std_logic; -- Forces all curves to be setup from buffers
			Reset_Run_Curves_x		: out std_logic; -- Initializes Run Mode DDS Curves
			
			Update_Trigger_x			: out std_logic; -- Trigger the Parameter Update Process
			Curve_Req_Trigger_x		: out std_logic; -- Trigger the Curve Request/Update Process
			Curve_Intrp_Trigger_x 	: out std_logic; -- Trigger the Curve Interpolation Process
			Bootup_Trigger_x			: out std_logic; -- Trigger the Flash Restore Process
			Backup_Trigger_x			: out std_logic; -- Trigger the Flash Store Process
			Flash_Bus_Permit_x		: out std_logic; -- Signal Flash to continue the Store Process
			DDS_Ready_x					: out std_logic;
			
			Run_Curves_x				: out std_logic;
			Update_DDS_Pulse_x		: out std_logic;
			Rewind_Curves_x			: out std_logic;
			Clear_Debug_FIFO			: out std_logic
						
		);
	end DDS_App_Control_Mux;
	
architecture main of DDS_App_Control_Mux is	



begin	
	
	process(VME_control_ena, VME_Ctrl, Verify_All, Set_CTBFREE_Sem, Preset_Curve_Status,
			Reset_Run_Curves, Update_Trigger, Curve_Req_Trigger, Curve_Intrp_Trigger,
			Bootup_Trigger, Backup_Trigger, Flash_Bus_Permit, DDS_Ready, Run_Curves,
			DDS_1us_Update_Pulse, Rewind_Curves, Flash_Bus_Req)
	begin
		if  (VME_control_ena = '1') then
			Verify_All_x				<= VME_Ctrl(0);
	      Set_CTBFREE_Sem_x			<= VME_Ctrl(6);
	      Preset_Curve_Status_x 	<= VME_Ctrl(7);
	      Reset_Run_Curves_x		<= VME_Ctrl(10);
	      
	      Update_Trigger_x			<= VME_Ctrl(1);
	      Curve_Req_Trigger_x		<= VME_Ctrl(4);
	      Curve_Intrp_Trigger_x 	<= VME_Ctrl(5);
	      Bootup_Trigger_x			<= VME_Ctrl(2);
	      Backup_Trigger_x			<= VME_Ctrl(3);
	      Flash_Bus_Permit_x		<= Flash_Bus_Req;
	      DDS_Ready_x					<= '1';
	        
			Run_Curves_x				<= VME_Ctrl(9);
			Update_DDS_Pulse_x		<= VME_Ctrl(11);
			Rewind_Curves_x			<= VME_Ctrl(12);
			Clear_Debug_FIFO			<= VME_Ctrl(13);
						           
		else
			Verify_All_x				<= Verify_All;
	      Set_CTBFREE_Sem_x			<= Set_CTBFREE_Sem;
	      Preset_Curve_Status_x 	<= Preset_Curve_Status;
	      Reset_Run_Curves_x		<= Reset_Run_Curves;
	                                 
	      Update_Trigger_x			<= Update_Trigger;
	      Curve_Req_Trigger_x		<= Curve_Req_Trigger;
	      Curve_Intrp_Trigger_x 	<= Curve_Intrp_Trigger;
	      Bootup_Trigger_x			<= Bootup_Trigger;
	      Backup_Trigger_x			<= Backup_Trigger;
	      Flash_Bus_Permit_x		<= Flash_Bus_Permit;
	      DDS_Ready_x					<= DDS_Ready;
	        			    
			Run_Curves_x				<= Run_Curves;
			Update_DDS_Pulse_x		<= DDS_1us_Update_Pulse;
			Rewind_Curves_x			<= Rewind_Curves;            
			Clear_Debug_FIFO			<= VME_Ctrl(13);
						           
	     end if;                          
	end process;
end;