--------------------------------------------------------------------
Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all;  -- my defpower_up_stateions and entities

-------------------------------------------------------------------- 

entity Initial_Reset is
	port(clk      		: in std_logic;
		 Start    		: in std_logic;
		 Reset_n 		: out std_logic;
		 Reset_PLL		: out std_logic;
		 Reset_Lower_n	: out std_logic
		);
end Initial_Reset;
	
architecture main of Initial_Reset is	
-- Counter/Timer declarations --------------------------------------
signal got_cnt_flag	: std_logic;
signal time_mark 	: std_logic_vector(19 downto 0);
signal nclr_cnt		: std_logic;

signal reset_1_n    : std_logic;
signal sync_reg		: std_logic_vector(2 downto 0);

-- State Machine declarations ------------------------------------
type STATE_TYPE is 
(init, pll_reset, clear_count, hold_reset, run);
signal CS, NS  : STATE_TYPE;

begin -- main --------------------------------------------------------
		
-------------------------------------------------------------------
SUMCNT: counter  -- Timer
generic map(20)
port map ( 	clk=>clk, 
			clrn=>nclr_cnt, 
			ena=>'1', 
			load=>'0', 
			updn=>'1',
			ld_data  => (others => '0'),
			flag_cnt => time_mark,
			flag_out => got_cnt_flag,
			cnt_out  => open
		 );

-- State Machine define initialization ---------------------------
	sm_reg: process (clk, Start)	
	begin
		if(Start = '0')             then CS <= init;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, Start, got_cnt_flag)
	begin
		-- default signal values
		time_mark <= X"40000"; 
		nclr_cnt <= '1';
		Reset_PLL<= '1';
		reset_1_n  <= '0';
		-------------------------
		case CS is
			when init =>
				nclr_cnt <= '0';
				if(Start = '1') then 		NS <= pll_reset;
				else								NS <= init;
				end if;
				
			when pll_reset =>  
				time_mark <= X"00004"; 
				if(got_cnt_flag = '1') then 	NS <= clear_count;
				else                      		NS <= pll_reset;
				end if;
				
			when clear_count =>					NS <= hold_reset;
				nclr_cnt <= '0';
				
			when hold_reset   =>
				Reset_PLL<= '0'; -- inactive now
				time_mark <= X"40000"; -- 5.2ms @ 20ns clock
				if(got_cnt_flag = '1') then 	NS <= run;
				else                      		NS <= hold_reset;
				end if;
			    
			when run  =>							NS <= run;
				Reset_PLL<= '0'; -- inactive now
				reset_1_n  <= '1'; -- inactive now
				
			when others =>              		NS <= run;
		end case; 			
	end process;	
	
--------------------------------------------------------------------
-- Reset release timing:  The lower FPGA will resync the release
--                        of the reset sent to it by the upper FPGA.
--                        These register are meant to keep the release
--                        times for upper and lower close together.

process (clk, reset_1_n)
begin
	if(reset_1_n = '0') then sync_reg <= (others => '0');
	elsif rising_edge(clk) then
		sync_reg(0) <= '1';
		sync_reg(1) <= sync_reg(0);
		sync_reg(2) <= sync_reg(1);
	end if;
end process;

Reset_Lower_n	<= sync_reg(0); -- this is registered for 2 clocks in Lower FPGA.
Reset_n 		<= sync_reg(2); -- this is registered 2 more clocks here.

end main;

