Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

----------------------------------------------------------------
-- Placement of these 32 bit registers is controlled by the 
-- constant PARAM_UPPER_FIRST.  This constant is defined in
-- the included file DDS_VME_CONSTANTS.vhd.

entity Upper_Parameter_Regs_16 is 
	port( 	
			clk			: in std_logic;
			nreset		: in std_logic;
			
			Data_to_Mem : in std_logic_vector(31 downto 0);
			Addr_to_Mem	: in std_logic_vector(23 downto 0);
			WR			: in std_logic;	
			
			Param_Data_Out	: out std_logic_vector(31 downto 0);
			Param_0		: out std_logic_vector(31 downto 0);
			Param_1		: out std_logic_vector(31 downto 0);
			Param_2		: out std_logic_vector(31 downto 0);
			Param_3		: out std_logic_vector(31 downto 0);
			Param_4		: out std_logic_vector(31 downto 0);
			Param_5		: out std_logic_vector(31 downto 0);
			Param_6		: out std_logic_vector(31 downto 0);
			Param_7		: out std_logic_vector(31 downto 0);			        
			Param_8		: out std_logic_vector(31 downto 0);
			Param_9		: out std_logic_vector(31 downto 0);
			Param_A		: out std_logic_vector(31 downto 0);
			Param_B		: out std_logic_vector(31 downto 0);
			Param_C		: out std_logic_vector(31 downto 0);
			Param_D		: out std_logic_vector(31 downto 0);
			Param_E		: out std_logic_vector(31 downto 0);
			Param_F		: out std_logic_vector(31 downto 0)			        
		);
	end Upper_Parameter_Regs_16;
	
architecture main of Upper_Parameter_Regs_16 is	

signal ADDR 		: std_logic_vector(23 downto 0);

signal param_0_out	: std_logic_vector(31 downto 0);
signal param_1_out	: std_logic_vector(31 downto 0);
signal param_2_out	: std_logic_vector(31 downto 0);
signal param_3_out	: std_logic_vector(31 downto 0);
signal param_4_out	: std_logic_vector(31 downto 0);
signal param_5_out	: std_logic_vector(31 downto 0);
signal param_6_out	: std_logic_vector(31 downto 0);
signal param_7_out	: std_logic_vector(31 downto 0);

signal param_8_out	: std_logic_vector(31 downto 0);
signal param_9_out	: std_logic_vector(31 downto 0);
signal param_A_out	: std_logic_vector(31 downto 0);
signal param_B_out	: std_logic_vector(31 downto 0);
signal param_C_out	: std_logic_vector(31 downto 0);
signal param_D_out	: std_logic_vector(31 downto 0);
signal param_E_out	: std_logic_vector(31 downto 0);
signal param_F_out	: std_logic_vector(31 downto 0);

            
begin
	
	ADDR <= Addr_to_Mem;
	
	PARAMETER_REGISTERS: process(clk, nreset, Data_to_Mem, WR, ADDR)
	begin	
		if rising_edge(clk) then
			if (nreset = '0') then
				param_0_out <= (others => '0');
				param_1_out <= (others => '0');
				param_2_out <= (others => '0');
				param_3_out <= (others => '0');
				param_4_out <= (others => '0');
				param_5_out <= (others => '0');
				param_6_out <= (others => '0');
				param_7_out <= (others => '0');
				
				param_8_out <= (others => '0');
				param_9_out <= (others => '0');
				param_A_out <= (others => '0');
				param_B_out <= (others => '0');
				param_C_out <= (others => '0');
				param_D_out <= (others => '0');
				param_E_out <= (others => '0');
				param_F_out <= (others => '0');

			elsif(WR = '1') then
				case ADDR is
					when PARAM_UPPER_FIRST 		 => param_0_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 4  => param_1_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 8  => param_2_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 12 => param_3_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 16 => param_4_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 20 => param_5_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 24 => param_6_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 28 => param_7_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 32 => param_8_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 36 => param_9_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 40 => param_A_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 44 => param_B_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 48 => param_C_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 52 => param_D_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 56 => param_E_out <= Data_to_Mem;
					when PARAM_UPPER_FIRST + 60 => param_F_out <= Data_to_Mem;
					when others =>						 null;
				end case;				
			end if;
		end if;
	end process;
       
	DATA_OUT_MUX: 
	process(ADDR, param_0_out, param_1_out, param_2_out, param_3_out, 
					param_4_out, param_5_out, param_6_out, param_7_out,
					param_8_out, param_9_out, param_A_out, param_B_out, 
					param_C_out, param_D_out, param_E_out, param_F_out)
	begin
		case ADDR is
			when PARAM_UPPER_FIRST 		 => Param_Data_Out <= param_0_out;
			when PARAM_UPPER_FIRST + 4  => Param_Data_Out <= param_1_out;
			when PARAM_UPPER_FIRST + 8  => Param_Data_Out <= param_2_out;
			when PARAM_UPPER_FIRST + 12 => Param_Data_Out <= param_3_out;
			when PARAM_UPPER_FIRST + 16 => Param_Data_Out <= param_4_out;
			when PARAM_UPPER_FIRST + 20 => Param_Data_Out <= param_5_out;
			when PARAM_UPPER_FIRST + 24 => Param_Data_Out <= param_6_out;
			when PARAM_UPPER_FIRST + 28 => Param_Data_Out <= param_7_out;
			when PARAM_UPPER_FIRST + 32 => Param_Data_Out <= param_8_out;
			when PARAM_UPPER_FIRST + 36 => Param_Data_Out <= param_9_out;
			when PARAM_UPPER_FIRST + 40 => Param_Data_Out <= param_A_out;
			when PARAM_UPPER_FIRST + 44 => Param_Data_Out <= param_B_out;
			when PARAM_UPPER_FIRST + 48 => Param_Data_Out <= param_C_out;
			when PARAM_UPPER_FIRST + 52 => Param_Data_Out <= param_D_out;
			when PARAM_UPPER_FIRST + 56 => Param_Data_Out <= param_E_out;
			when PARAM_UPPER_FIRST + 60 => Param_Data_Out <= param_F_out;
			when others =>						 Param_Data_Out <= (others => '0');
		end case;								
	end process;

	Param_0 <= param_0_out;
	Param_1 <= param_1_out;
	Param_2 <= param_2_out;
	Param_3 <= param_3_out;
	Param_4 <= param_4_out;
	Param_5 <= param_5_out;
	Param_6 <= param_6_out;
	Param_7 <= param_7_out;
	
	Param_8 <= param_8_out;
	Param_9 <= param_9_out;
	Param_A <= param_A_out;
	Param_B <= param_B_out;
	Param_C <= param_C_out;
	Param_D <= param_D_out;
	Param_E <= param_E_out;
	Param_F <= param_F_out;

          
end;