Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

----------------------------------------------------------------
-- Placement of these 32 bit registers is controlled by the 
-- constant VME_CTRL_UPPER_FIRST.  This constant is defined in
-- the included file DDS_VME_CONSTANTS.vhd.

-- Version 2:  Made Register 7 bit 0 into a flag to signal the desire
-- to do a backup if a parameter or curve has changed.  If a
-- parameter or curve has not changed since the last backup the 
-- Enable_Backup bit will stay active until a backup occurs.

entity Upper_VME_Ctrl_Regs_v2 is 
	port( 	
			clk			: in std_logic;
			nreset		: in std_logic;
			
			Data_to_Mem 	: in std_logic_vector(31 downto 0);
			Addr_to_Mem		: in std_logic_vector(23 downto 0);
			WR					: in std_logic;
			status_reg_0	: in std_logic_vector(31 downto 0);	
			Backup_Started	: in std_logic;
			
			reg_Data_Out	: out std_logic_vector(31 downto 0);
			reg_0				: out std_logic_vector(31 downto 0);
			reg_1				: out std_logic_vector(31 downto 0);
			reg_2				: out std_logic_vector(31 downto 0);
			reg_3				: out std_logic_vector(31 downto 0);
			reg_4				: out std_logic_vector(31 downto 0);
			reg_5				: out std_logic_vector(31 downto 0);
			reg_6				: out std_logic_vector(31 downto 0);
			reg_7				: out std_logic_vector(31 downto 0)
		);
	end Upper_VME_Ctrl_Regs_v2;
	
architecture main of Upper_VME_Ctrl_Regs_v2 is	

signal ADDR 		: std_logic_vector(23 downto 0);

signal reg_0_out	: std_logic_vector(31 downto 0);
signal reg_1_out	: std_logic_vector(31 downto 0);
signal reg_2_out	: std_logic_vector(31 downto 0);
signal reg_3_out	: std_logic_vector(31 downto 0);
signal reg_4_out	: std_logic_vector(31 downto 0);
signal reg_5_out	: std_logic_vector(31 downto 0);
signal reg_6_out	: std_logic_vector(31 downto 0);
signal reg_7_out	: std_logic_vector(31 downto 0);
            
begin
	
	ADDR <= Addr_to_Mem;
	
 	PARAMETER_REGISTERS: process(clk, nreset, WR, ADDR, Data_to_Mem)
	begin	
		if rising_edge(clk) then
			if (nreset = '0') then
				reg_0_out <= (others => '0');
				reg_1_out <= (others => '0');
				reg_2_out <= (others => '0');
				reg_3_out <= (others => '0');
				reg_4_out <= (others => '0');
				reg_5_out <= (others => '0');
				reg_6_out <= (others => '0');
				reg_7_out <= (others => '0');
			elsif(WR = '1') then
				case ADDR is
					when VME_CTRL_UPPER_FIRST 			=> reg_0_out <= Data_to_Mem;
					when VME_CTRL_UPPER_FIRST + 4 	=> reg_1_out <= Data_to_Mem;
					when VME_CTRL_UPPER_FIRST + 8    => reg_2_out <= Data_to_Mem;
					when VME_CTRL_UPPER_FIRST + 12   => reg_3_out <= Data_to_Mem;
					when VME_CTRL_UPPER_FIRST + 16   => reg_4_out <= Data_to_Mem;
					when VME_CTRL_UPPER_FIRST + 20   => reg_5_out <= Data_to_Mem;
					when VME_CTRL_UPPER_FIRST + 24   => reg_6_out <= Data_to_Mem;
					when VME_CTRL_UPPER_FIRST + 28   => reg_7_out <= Data_to_Mem;
					when others 							=> NULL;        
				end case;
			elsif(Backup_Started = '1') 			 then reg_7_out(0) <= '0';
			end if;
		end if;
	end process;
      
	DATA_OUT_MUX: 
	process(ADDR, reg_0_out, reg_1_out, reg_2_out, reg_3_out, 
			reg_4_out, reg_5_out, reg_6_out, reg_7_out, status_reg_0)
	begin
				case ADDR is
					when VME_CTRL_UPPER_FIRST 			=> reg_Data_Out <= reg_0_out;
					when VME_CTRL_UPPER_FIRST + 4 	=> reg_Data_Out <= reg_1_out;
					when VME_CTRL_UPPER_FIRST + 8    => reg_Data_Out <= reg_2_out;
					when VME_CTRL_UPPER_FIRST + 12   => reg_Data_Out <= reg_3_out;
					when VME_CTRL_UPPER_FIRST + 16   => reg_Data_Out <= reg_4_out;
					when VME_CTRL_UPPER_FIRST + 20   => reg_Data_Out <= reg_5_out;
					when VME_CTRL_UPPER_FIRST + 24   => reg_Data_Out <= reg_6_out;
					when VME_CTRL_UPPER_FIRST + 28   => reg_Data_Out <= reg_7_out;
					when VME_STAT_UPPER			   	=> reg_Data_Out <= status_reg_0;
					when others 							=> reg_Data_Out <= (others => '0');       
				end case;
	end process;

	reg_0 <= reg_0_out;
	reg_1 <= reg_1_out;
	reg_2 <= reg_2_out;
	reg_3 <= reg_3_out;
	reg_4 <= reg_4_out;
	reg_5 <= reg_5_out;
	reg_6 <= reg_6_out;
	reg_7 <= reg_7_out;
          
end;