Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

-------------------------------------------------------------------------------
-- MODULE DESCRIPTION, May 11, 2010

entity VME_Addr_Decoder_v7 is 
	port( 
			Addr_to_Mem		: in std_logic_vector(23 downto 0);
			WR_Pulse		: in std_logic;			
			VME_WRITE_ACTIVE: in std_logic;

			SM_Data_Out		: in std_logic_vector(31 downto 0);
			Setup_Data_Out	: in std_logic_vector(31 downto 0);
			PP_Data_Out		: in std_logic_vector(31 downto 0);
			PA_Data_Out		: in std_logic_vector(31 downto 0);
			PB_Data_Out		: in std_logic_vector(31 downto 0);
			Ctrl_Reg_Out	: in std_logic_vector(31 downto 0);
			Monitor_Data_Out : in std_logic_vector(31 downto 0);
			IO_Reg_Out		: in std_logic_vector(31 downto 0);
			Lower_Data_Out	: in std_logic_vector(31 downto 0);
			
			Data_Out		: out std_logic_vector(31 downto 0);		
			WR_SM			: out std_logic;
			WR_PP			: out std_logic;
			WR_PA			: out std_logic;
			WR_PB			: out std_logic;
			WR_SETUP		: out std_logic
			        
		);
	end VME_Addr_Decoder_v7;
	
architecture main of VME_Addr_Decoder_v7 is
	
signal WR   : std_logic;
signal ADDR : std_logic_vector(23 downto 0);

begin	
	
	WR <= WR_Pulse;
	ADDR <= Addr_to_Mem;	
	
	process (ADDR, Lower_Data_Out, SM_Data_Out, Setup_Data_Out,
			 PP_Data_Out, PA_Data_Out, PB_Data_Out, Ctrl_Reg_Out, 
			 Monitor_Data_Out, IO_Reg_out, VME_WRITE_ACTIVE )
	begin
			Data_Out <= X"0000BAD7";
		
		if(VME_WRITE_ACTIVE = '1') 		then 			Data_Out <= Lower_Data_Out;
	
		elsif(ADDR < X"014000") then  Data_Out <= SM_Data_Out;
		elsif(ADDR < X"014400") then  Data_Out <= Setup_Data_Out;   
		elsif(ADDR < X"014800") then  NULL;
		elsif(ADDR < X"015000") then  Data_Out <= PP_Data_Out; 
		elsif(ADDR < X"01A000") then  Data_Out <= PA_Data_Out;
		elsif(ADDR < X"01F000") then  Data_Out <= PB_Data_Out;
		elsif(ADDR < X"01F100") then  NULL;
		elsif(ADDR < X"01F200") then  Data_Out <= Ctrl_Reg_Out;
		elsif(ADDR < X"01F300") then  Data_Out <= Monitor_Data_Out;
		elsif(ADDR < X"020000") then  Data_Out <= IO_Reg_Out;
		elsif(ADDR < X"080000") then  Data_Out <= Lower_Data_Out;
		end if;

	end process;
	
	process (ADDR, WR )
	begin
				WR_SM		<= '0';
            WR_SETUP	<= '0';
            WR_PP		<= '0';	
            WR_PA		<= '0';	
            WR_PB		<= '0';	
			
		if   (ADDR < X"014000") then     WR_SM <= WR;
		elsif(ADDR < X"014400") then  WR_SETUP <= WR;
		elsif(ADDR < X"014800") then 	        NULL;
		elsif(ADDR < X"015000") then     WR_PP <= WR;
		elsif(ADDR < X"01A000") then     WR_PA <= WR;
		elsif(ADDR < X"01F000") then     WR_PB <= WR;
		elsif(ADDR < X"01F100") then 	   NULL;
		elsif(ADDR < X"01F200") then  	NULL;
		elsif(ADDR < X"01F300") then  	NULL;
		elsif(ADDR < X"020000") then    	NULL;	 
		elsif(ADDR < X"080000") then  	NULL; 	
		end if;

	end process;
	
end;